local rc = require 'rclib'

-- 0° is East; look North instead (90°)
local x, y, angle = 0, 0, math.rad(90)

local speed = 2 -- m/s
local omega = 1.3 -- rad/s

-- window width, height, arrow centre x/y, viewport size (width=height), arrow
local ww, wh, acx, acy, vs, arrow
local ab1x, ab1y, ab2x, ab2y, ab3x, ab3y, ab4x, ab4y
local pressed1, pressed2, pressed3, pressed4

function love.load()
  local img = love.graphics.newImage('image1.jpg')
  rc.init(img)
  rc.rect3d(-6, 6, -1,
            -6,-6, -1,
             6, 6, -1,
              0, 0, 640, 480, img:getDimensions())
  rc.rect3d( 0, 5, 1,
             0, 5, -1,
             2, 5, 1,
              0, 0, 320, 240, img:getDimensions())


  rc.setCamera(x, y, 0, angle)

  arrow = love.graphics.newImage('arrow.png')
  love.resize()
end

local function pressed(x1, y1, xm, ym)
  return x1 > xm - 40 and x1 < xm + 40 and y1 > ym - 40 and y1 < ym + 40
end

function love.mousepressed(x, y, b)
  if b == 1 then
    pressed1 = pressed(ab1x, ab1y, x, y)
    pressed2 = pressed(ab2x, ab2y, x, y)
    pressed3 = pressed(ab3x, ab3y, x, y)
    pressed4 = pressed(ab4x, ab4y, x, y)
  end
end

function love.mousereleased(x, y, b)
  pressed1 = false
  pressed2 = false
  pressed3 = false
  pressed4 = false
end

function love.update(dt)
  if love.keyboard.isDown("left") or pressed1 then
    angle = (angle + omega * dt) % (2*math.pi)
  end
  if love.keyboard.isDown("right") or pressed2 then
    angle = (angle - omega * dt) % (2*math.pi)
  end
  if love.keyboard.isDown("up") or pressed3 then
    x, y = x + math.cos(angle) * speed * dt, y + math.sin(angle) * speed * dt
  end
  if love.keyboard.isDown("down") or pressed4 then
    x, y = x - math.cos(angle) * speed * dt, y - math.sin(angle) * speed * dt
  end
  rc.setCamera(x, y, 0, angle)
end

function love.resize()
  -- Arrow button handling
  ww, wh = love.graphics.getDimensions()
  acx, acy = arrow:getDimensions()
  acx, acy = acx*0.5, acy*0.5
  vs = math.floor(wh*0.5) * 2
  -- at least 80 pixels on the side
  if vs >= ww - 80 then vs = math.floor((ww - 80)*0.5) * 2 end
  ab1x = (ww - vs - 80) * 0.5
  ab1y = wh * 0.5
  ab2x = (ww + vs + 80) * 0.5
  ab2y = ab1y
  ab3x = ab1x
  ab3y = ab1y - 80
  ab4x = ab1x
  ab4y = ab1y + 80
end

function love.draw()
  -- Draw side buttons
  love.graphics.setShader()
  love.graphics.draw(arrow, ab1x, ab1y, math.rad(-90), 1, 1, acx, acy)
  love.graphics.draw(arrow, ab2x, ab2y, math.rad(90), 1, 1, acx, acy)
  love.graphics.draw(arrow, ab3x, ab3y, 0, 1, 1, acx, acy)
  love.graphics.draw(arrow, ab4x, ab4y, math.pi, 1, 1, acx, acy)

  love.graphics.setScissor((ww - vs)*0.5, (wh-vs)*0.5, vs, vs)
  love.graphics.setShader(rc.shader)
  rc.draw()
  love.graphics.setScissor()
end


function love.keypressed(k) if k == "escape" then love.event.quit() end end
